#!/usr/bin/python

import csv, os, re, sys
from optparse import OptionParser

from pysqlite2 import dbapi2 as sqlite
#import sqlite

class SQLDialect(csv.Dialect):
    delimiter = '|'
    quotechar = '"'
    escapechar = '\\'
    doublequote = False
    #quoting = csv.QUOTE_ALL
    quoting = csv.QUOTE_NONNUMERIC
    skipinitialspace = False
    #lineterminator = '\r\n'
    lineterminator = '\n'

#------------------------------------------
usage = "usage: %prog -i [nbefile] [options]"
version = "1.0 by Garrett Gee <garrett - at - garrettgee.com>"
parser = OptionParser(usage=usage,version=version)
parser.add_option("-i", "--inputfile", dest="inputfile", help="nbe input file")
parser.add_option("-o", "--outputfile", dest="outputfile", help="if not defined, will create dat based on filename")
parser.add_option("-v", "--verbose", action="store_true", dest="verbose", help="")
parser.add_option("-d", "--debug", action="store_true", dest="debug", help="")

(options, args) = parser.parse_args()

if (not (options.inputfile)) or (not os.path.isfile(options.inputfile)):
    parser.print_help()
    sys.exit()

nbefile = options.inputfile
dbfile = options.outputfile
verbose = options.verbose
debug = options.debug
#------------------------------------------
if not (dbfile):
    (dbfile,ext) = os.path.splitext(nbefile)
    dbfile = dbfile + ".dat"
    
if verbose:
    print "Inputfile: " + nbefile    
    print "Outputfile: " + dbfile

if os.path.isfile(dbfile):
    ##this should be a choice box
    print "Removing old " + dbfile
    os.remove(dbfile)
#------------------------------------------
cx = sqlite.connect(dbfile)
cu = cx.cursor()

#------------------------------------------
if verbose:
    print "creating nessus table"
cu.execute("create table nessus ( \
id INTEGER PRIMARY KEY, \
IP VARCHAR(15), \
service VARCHAR(20), \
port INTEGER, \
protocol VARCHAR(4), \
pluginid INTEGER, \
sevlevel VARCHAR(20), \
notes VARCHAR(1000) \
)")
#------------------------------------------
def splitonslash(string):
    string = re.sub("\(", "", string)
    string = re.sub("\)", "", string)
    (key1, key2) = string.split("/",1)
    return key1, key2
#------------------------------------------
def parserow3(list):
    tmprow3 = list.split(" ",1)
    if len(tmprow3) > 1:
        (port, protocol) = splitonslash(tmprow3[1])
        service = tmprow3[0]
    else:
        (service, protocol) = splitonslash(tmprow3[0])
        port = ""
    return service, port, protocol
#------------------------------------------
file = open(nbefile, "r")
whole_file = csv.reader(file, dialect=SQLDialect)
for row in whole_file:
    if len(row) == 0:
        pass
    else:
        if row[0] == "timestamps":
            if row[3] == "host_start":
                cu.execute("insert into nessus (IP) values (?)", (row[2],))
        if row[0] == "results":
            if len(row) > 4:
                (service, port, protocol) = parserow3(row[3])
                values = row[2] + " " + row[3] + " " + row[4] + " " + row[5] + " " + row[6] + "\n"
                cu.execute("insert into nessus (IP, service, port, protocol, pluginid, sevlevel, notes) \
                    values (?, ?, ?, ?, ?, ?, ?)", \
                    (row[2], service, port, protocol, row[4], row[5], row[6]))
            else:
                (service, port, protocol) = parserow3(row[3])
                values = row[2] + " " + row[3] + "\n"
                cu.execute("insert into nessus (IP, service, port, protocol) \
                    values (?, ?, ?, ?)", \
                    (row[2], service, port, protocol))
            if debug:
                print service + " " + port + " " + protocol
                print values
file.close()
#------------------------------------------
if verbose:
    print "creating IP and hostname index"
cu.execute("CREATE INDEX IP_index ON nessus (IP, port)")
#------------------------------------------
cx.commit()
#------------------------------------------
